"""
Pykrete setup
Author: Shai Bennathan - shai.bennathan@gmail.com
(C) 2020
"""

from .project_test_command import ProjectTestCommand
from .self_test_command import SelfTestCommand

__all__ = ['ProjectTestCommand', 'SelfTestCommand']
